import { readFileSync } from "fs";
import { resolve, sep } from "path";
import prettier from "prettier";
export default function createTaroRouteConfig({ home }) {
    return (source) => {
        // console.log("source", JSON.stringify(source, null, 2))
        // FIXME: source : FileType
        const routeList = source[0].filesInfo.filter((it) => {
            return it.base === "index.tsx" && !/components/.test(it.dir);
        });
        // 生成所有 Name
        const routeNames = routeList.map((it) => {
            const pathLayer = it.dir.split(sep);
            // TODO: 处理路由别名；实际代码使用无意义的名字
            // const annotation = getAnnotation(readFileSync(it.path, "utf-8"))
            // console.log("%c debug", "color:white;background: rgb(83,143,204);padding:4px", annotation, it.path)
            let routeName = pathLayer[pathLayer.length - 1];
            try {
                const annotation = getAnnotation(readFileSync(it.path, "utf-8"));
                const aliasMap = annotation === null || annotation === void 0 ? void 0 : annotation.get("alias");
                if (aliasMap) {
                    aliasMap.forEach((element) => {
                        const obj = element.split("=");
                        if ((obj === null || obj === void 0 ? void 0 : obj[0]) === "routeAlias" && (obj === null || obj === void 0 ? void 0 : obj[1])) {
                            routeName = obj === null || obj === void 0 ? void 0 : obj[1];
                        }
                    });
                }
            }
            catch (error) {
                console.log(error);
            }
            return { name: routeName, path: it.path };
        });
        // console.log("routeNames", sep, routeNames)
        const codePart1 = `import { RouteList } from 'taro-route';

    export type Name = ${routeNames.map((it) => `"${it.name}"`).join(" | ")};

    `;
        // 获取 route： pages/package-xx/[name]/index.tsx
        const packageRouteList = routeList.filter((it) => /pages\/package[A-Z]+\/[a-zA-Z]+/.test(it.path));
        const packageRoute = packageRouteList.map((it) => {
            var _a;
            const pathLayer = it.dir.split(sep);
            const name = (_a = routeNames.find((item) => item.path === it.path)) === null || _a === void 0 ? void 0 : _a.name;
            const packageName = `${pathLayer[pathLayer.length - 2]}`;
            const routeName = pathLayer[pathLayer.length - 1];
            return {
                name,
                path: `/pages/${packageName}/${routeName}/index`,
                isTab: false,
            };
        });
        // 获取 route: pages/tab/[name]/index.tsx
        const tabRouteList = routeList.filter((it) => /pages\/tab\/[a-zA-Z]+/.test(it.path));
        const tabRoute = tabRouteList.map((it) => {
            var _a;
            const pathLayer = it.dir.split(sep);
            const routeName = pathLayer[pathLayer.length - 1];
            const name = (_a = routeNames.find((item) => item.path === it.path)) === null || _a === void 0 ? void 0 : _a.name;
            return {
                name,
                path: `/pages/tab/${routeName}/index`,
                isTab: true,
            };
        });
        // 获取 route: pages/[name]/index.tsx
        const normalRouteList = routeList.filter((it) => !packageRouteList.includes(it) && !tabRouteList.includes(it));
        const normalRoute = normalRouteList.map((it) => {
            var _a;
            const pathLayer = it.dir.split(sep);
            const routeName = pathLayer[pathLayer.length - 1];
            const name = (_a = routeNames.find((item) => item.path === it.path)) === null || _a === void 0 ? void 0 : _a.name;
            return {
                name,
                path: `/pages/${routeName}/index`,
                isTab: false,
            };
        });
        // console.log('packageRouteList', JSON.stringify(packageRouteList, null, 2))
        // console.log('tabRouteList', JSON.stringify(tabRouteList, null, 2))
        // console.log('normalRouteList', normalRoute)
        // console.log('packageRouteList', packageRoute)
        // console.log('tabRouteList', tabRoute)
        const routeConfigList = [...packageRoute, ...tabRoute, ...normalRoute];
        const codePart2 = `export const routeList: RouteList<Name> = ${JSON.stringify(routeConfigList)};`;
        const pages = routeConfigList.map((it) => it.path.slice(1, it.path.length));
        pages.unshift(home);
        const pageSet = new Set(pages);
        const codePart3 = `export const pages = ${JSON.stringify(Array.from(pageSet))}`;
        const generatedCode = codePart1 + codePart2 + codePart3;
        const formateCode = prettier.format(generatedCode, {
            printWidth: 80,
            tabWidth: 2,
            trailingComma: "all",
            parser: "babel",
        });
        return [
            {
                pathname: resolve("./src", "router/generated.ts"),
                code: formateCode,
            },
        ];
    };
}
//新增
function cacheMap(comments) {
    const map = new Map();
    const reg = /^@([a-z]*) /;
    comments.forEach((comment) => {
        var _a;
        const regComment = reg.exec(comment);
        if ((regComment === null || regComment === void 0 ? void 0 : regComment.length) === 0)
            return;
        const readyComment = comment.replace(reg, "");
        const key = ((_a = regComment === null || regComment === void 0 ? void 0 : regComment[0]) !== null && _a !== void 0 ? _a : "")
            .trim()
            .replace("@", "");
        const mapValue = map.get(key);
        if (mapValue) {
            mapValue.add(readyComment);
            return;
        }
        const set = new Set();
        set.add(readyComment);
        map.set(key, set);
    });
    return map;
}
//新增
function getQuery(comment) {
    comment = comment.replace(/\*\/$/, ""); //  */替换成""
    let splitComment = comment.split("@");
    splitComment = splitComment
        .slice(1, splitComment.length)
        .map((val) => ("@" + val.replace(/((\* $)|(\* ))/gm, "")).trim());
    return cacheMap(splitComment);
}
function getAnnotation(content) {
    var _a;
    const execContent = /\/\*(\s|.)*?\*\//.exec(content);
    if ((execContent === null || execContent === void 0 ? void 0 : execContent.length) === 0)
        return undefined;
    const comment = (_a = execContent === null || execContent === void 0 ? void 0 : execContent[0]) !== null && _a !== void 0 ? _a : "";
    return getQuery(comment);
}
